var mouseControls = mouseControls || {};

//create objects used to raycaste with a mouse click;
mouseVector2 = new THREE.Vector2();
mouseRay = new THREE.Raycaster();

///////////////////////////////////////////////////////////////////
// WALL PLACEMENT
///////////////////////////////////////////////////////////////////

mouseControls.checkMouseDownForWallPlacement = function( event ){

	// we'll assume the the vertices will be stored in cells.previousSelection

	// first will see if cells.previousSelection is populated with any vertices
	if( cells.previousSelection.length > 0 ){

		// we're in luck
		// next we'll check if the wall already exists at this location
		if( walls.lookUpArray[ cells.previousSelection[0] ] ){

			// the wall already exists.  check if the user wants to delete the wall
			if( event.buttons == 1 ){

				// no further action required

			} else {

				// the user has right clicked on the wall, so we're going to remove it 
				// and add its vertice location back to the availability pool

				var indexToClear = walls.bufferLink[ cells.previousSelection[0] ];

				// reset vertices to 0,0,0
				walls.geometry.vertices[ indexToClear ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+1 ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+2 ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+3 ].set( 0 , 0 , 0 );

				// Now we update the travelTo array to allow movement.
				mouseFunctions.updateCell( cells, true , "travelTo" );

				// add index position to the availability pool
				walls.availableIndexes.push( indexToClear );

				// set lookUpArray to false
				walls.lookUpArray[ cells.previousSelection[0] ] = false;

				// now that we've set the location of the vertices we'll need to update the geometry.
				// the faces will automatically shrink into place as each set of 2 faces is connected to
				// 4 vertices.
				walls.geometry.computeFaceNormals();
				walls.geometry.verticesNeedUpdate = true;

			}

		} else {

			// now we'll see if the walls bufferIndex has any more room
			if( walls.bufferIndex.vertices == walls.bufferIndex.maxVertices ){

				// the maximum number of walls has ben created, check to see if there
				// is any room in the availableIndexes array
				if( walls.availableIndexes.length == 0 ){

					// maximum walls created.
					// no further action required

				} else {

					// create a wall using an index that has been freed up.
					// but first we'll check to see if creating this wall blocks
					// the passage to an exit.
					if( mouseFunctions.checkForClearPassage( cells ) ){

						mouseFunctions.createWall( walls.availableIndexes[0] );

						// add an entry into buffer Link
						walls.bufferLink[ cells.previousSelection[0] ] = walls.availableIndexes[0];

						// remove available Index
						walls.availableIndexes.splice( 0 , 1 );
					} 					

				}

			} else {



				// create a wall using a new index
				// but first we'll check to see if creating this wall blocks
				// the passage to an exit.
				if( mouseFunctions.checkForClearPassage( cells ) ){

					mouseFunctions.createWall( walls.bufferIndex.vertices );

					// add an entry into buffer Link
					walls.bufferLink[ cells.previousSelection[0] ] = walls.bufferIndex.vertices;

					// lastly, update the buffer index
					walls.bufferIndex.vertices +=4;
				} 
			}
		}
	}
}

mouseControls.checkMouseMoveForWallPlacement = function( event ){

	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	if( intersect.length > 0 ){

		// reset colors of all previous lines that were selected
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of lines.
		cells.previousSelection = [];

		// change the color of the line we've selected
		var face = intersect[ 0 ].face;
		var vectorList = cells.list[ face.index ].lineVertices;

		// highlight the closest line if within a certain distance.
		// this way it will seem a little more intuitive when selecting a line.
		var closestDistance = 10;
		var temporaryDistance = 0;
		var verticesIndex = -1;
		var allowedDistance = 0.1065; // This number seems to have the right feel to it.

		// we're going to calculate the distance from the intersected point 
		// to each set of vertices used to draw the grid lines, only if it's short enough will
		// we allow a selection.  
		for( var i=0; i<vectorList.length; i+=2 ){
			temporaryDistance = grid.geometry.vertices[ vectorList[i] ].distanceTo( intersect[0].point ) + 
								grid.geometry.vertices[ vectorList[ i+1 ] ].distanceTo( intersect[0].point );
			if( temporaryDistance < closestDistance && temporaryDistance <= allowedDistance ){
				closestDistance = temporaryDistance;
				verticesIndex = vectorList[i];

				// we're going to record the side that we're going to create a wall at
				// this will come in handy when we update cells.list[].travelTo later.
				cells.info.cellID = face.index;
				cells.info.side = i*0.5;
			}
		}
		
		// check to see if a line has been found.
		if( verticesIndex > -1 ){

			// now we'll highlight the line closest to the intersection point.
			grid.geometry.colors[ verticesIndex ].setRGB( 1 , 1 , 0 );
			grid.geometry.colors[ verticesIndex + 1 ].setRGB( 1 , 1 , 0 );			
			cells.previousSelection.push( verticesIndex );
			cells.previousSelection.push( verticesIndex + 1 );	
		} else {
			// rest cells.side.
			cells.info.cellID = -1;
			cells.info.side = -1;
		}

	} else {

		// reset colors of all previous lines that were selected
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of lines.
		cells.previousSelection = [];

	}

	grid.geometry.colorsNeedUpdate = true;

	if( !mouseFunctions.checkForClearPassage( cells ) ){

		// turn the color of the line purple to indicate that the wall can't be built
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 1 , 0 , 0 );
		}
	}
}

///////////////////////////////////////////////////////////////////
// TOWER PLACEMENT
///////////////////////////////////////////////////////////////////

/*
mouseControls.checkMouseDownForTowerPlacement = function( event ){


	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	// check to see if we intersected the gridTargetMesh
	if( intersect.length > 0 ){

		// reset colors of all previous cells that were selected
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

		// change the color of the square we've selected
		var face = intersect[ 0 ].face;
		var vectorList = cells.list[ face.index ].lineVertices;

		for( var i=0; i<vectorList.length; i++ ){
			grid.geometry.colors[ vectorList[i] ].setRGB( 0 , 0 , 1 );
			cells.previousSelection.push( vectorList[ i ] );
		}

		grid.geometry.colorsNeedUpdate = true;

	} else {

		// if we click outside the play area we'll reset the colors of the last selected square.
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

		grid.geometry.colorsNeedUpdate = true;

	}
}
*/

mouseControls.checkMouseMoveForTowerPlacement = function( event ){


	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	// check to see if we intersected the gridTargetMesh
	if( intersect.length > 0 ){

		// reset colors of all previous cells that were selected
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

		// change the color of the square we've selected
		var face = intersect[ 0 ].face;
		var vectorList = [];
		
		// first we'll check to see if this spot can have a tower.  Is there passage allowed to the 
		// surrounding cells in question?
		var canBuildTowerHere = false;
		// when we draw the temporary square we'll use the following color scheme
		var color = { r:1, g:0, b:0 };
		if( cells.list[ face.index ].travelTo[ 0 ] && cells.list[ face.index ].travelTo[ 1 ] && cells.list[ face.index ].travelTo[ 2 ] ){
			canBuildTowerHere = true;
			color = { r:0, g:1, b:1 };
		}

		// we're going to collect the face indexes of four squares 2x2
		vectorList[0] = cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].lineVertices;
		vectorList[1] = cells.list[ cells.list[ face.index ].surroundingCells[ 1 ] ].lineVertices;
		vectorList[2] = cells.list[ cells.list[ face.index ].surroundingCells[ 2 ] ].lineVertices;
		vectorList[3] = cells.list[ face.index ].lineVertices;

		// then we're going to only draw their corners.
		for( var i=0, a=0, b=6; i<4; i++ ){
			if( vectorList[ i ][ a ] != undefined ){
				grid.geometry.colors[ vectorList[ i ][ a ] ].setRGB( color.r , color.g , color.b );
				cells.previousSelection.push( vectorList[ i ][ a ] );
				grid.geometry.colors[ vectorList[ i ][ a+1 ] ].setRGB( color.r , color.g , color.b );
				cells.previousSelection.push( vectorList[ i ][ a+1 ] );
				grid.geometry.colors[ vectorList[ i ][ b ] ].setRGB( color.r , color.g , color.b );
				cells.previousSelection.push( vectorList[ i ][ b ] );
				grid.geometry.colors[ vectorList[ i ][ b+1 ] ].setRGB( color.r , color.g , color.b );
				cells.previousSelection.push( vectorList[ i ][ b+1 ] );
				a+=2;
				b+=2;
				if( b > 6 ){ b = 0; }
			}
		}

	} else {

		// if we click outside the play area we'll reset the colors of the last selected square.
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 0.3 , 0.3 , 0.3 );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

	}

	grid.geometry.colorsNeedUpdate = true;
}