var mouseFunctions = mouseFunctions || {};

mouseFunctions.checkForClearPassage = function( cells ){

	var aCell;

	// set all currentIteration arrays to their travelTo state.
	for( var i=0, ii=cells.list.length; i<ii; i++ ){
		aCell = cells.list[ i ];
		for( var j=0; j<8; j++ ){
			aCell.currentIteration[ j ] = aCell.travelTo[ j ];
		}
	}

	// first we'll pretend that the wall has been added.
	// then we'll test to see if the path is clear
	updateCell( cells , false , "currentIteration" );

	// path will be an array that we add to.  At each iteration of our Flood Fill pathfinding algorithm
	// the path will copy its self for each subsequent iteration and add a unique bit to each new construct.  
	// Each Path item will contain an array of all spaces it has traveled to and begin with the overall distance.
	// For now the path will start at the last square.
	// the first entry in the path array will be an object indicating whether or not an end goal has been found.
	// and what the indexed position is for the shortest path and the length of the shortest path.
	var path = [ { goal: false , index: -1, length: 10000 } , [ 0 , ( cells.list.length - 1 ) ] ];

	iterateThroughAllDirections( cells , path );

	return path[0].goal;
}

mouseFunctions.createWall = function( index ){

	// the wall does not exist and there is room to build.  
	// we'll create one at this location.
	walls.lookUpArray[ cells.previousSelection[0] ] = true;

	// assign two temporary placeholder variables
	var v1 = grid.geometry.vertices[ cells.previousSelection[0] ];
	var v2 = grid.geometry.vertices[ cells.previousSelection[1] ];

	walls.geometry.vertices[ index ].set( v1.x , v1.y , v1.z );
	walls.geometry.vertices[ index+1 ].set( v1.x , v1.y+0.03 , v1.z );

	walls.geometry.vertices[ index+2 ].set( v2.x , v2.y , v2.z );
	walls.geometry.vertices[ index+3 ].set( v2.x , v2.y+0.03 , v2.z );

	// Now we update the travelTo array to restrict movement.
	updateCell( cells , false , "travelTo" );

	// now that we've set the location of the vertices we'll need to update the geometry.
	// the faces will automatically stretch into place as each set of 2 faces is connected to
	// 4 vertices.
	walls.geometry.verticesNeedUpdate = true;
	walls.geometry.facesNeedUpdate = true;
	walls.geometry.normalsNeedUpdate = true;
	walls.geometry.computeFaceNormals();
	walls.geometry.computeBoundingSphere();
	walls.updateMatrixWorld();
}

mouseFunctions.updateCell = function( cells , bool , array ){

	updateCell( cells , bool , array );	
}

function iterateThroughAllDirections( cells , path ){

	var aCell, bCell, newPath=[];

	// the reciprical cell position of connected cell.
	var cellReciprical = [ 4 , 5 , 6 , 7 , 0 , 1 , 2 , 3 ];

	// cycle through all paths.
	for( var a=1, aa=path.length; a<aa; a++ ){

		aCell = cells.list[ path[ a ][ path[ a ].length - 1 ] ];

		// we'll copy over the length of this path segment into the new array newPath.
		newPath = [ path[ a ][ 0 ] ];

		// we're going to store the length of the path in the zero index of newPath
		// if we travel from left/right or up/down we'll increment by 1
		// if diagonally, we'll increment by 1.415

		// check for passage to each of the eight connected cells
		for( var i=0; i<8; i++ ){
			
			bCell = cells.list[ aCell.surroundingCells[ i ] ];

			// if passage is allowed, copy the indexed path array and continue.
			if( aCell.currentIteration[ i ] ){

				newPath = [];

				for( var j=0, jj=path[ a ].length; j<jj; j++ ){
					newPath[ j ] = path[ a ][ j ];
				}

				// we'll add to the length of the current iteration/newPath
				if( i % 2 == 0 ){
					newPath[ 0 ] ++;
				} else {
					newPath[ 0 ] += 1.415;
				}

				// adjust currentIteration for both to and from cells to false;
				aCell.currentIteration[ i ] = false;
				bCell.currentIteration[ cellReciprical[ i ] ] = false;

				//check to see if this cell is a goal cell
				if( cells.list[ aCell.surroundingCells[ i ] ].goal ){

					// we've found a path to the exit.
					path[ 0 ].goal = true;

					// is this newly found path shorter than any previous ones?
					if( newPath[ 0 ] < path[ 0 ].length ){
						path[ 0 ].index = a;
						path[ 0 ].length = newPath[ 0 ];
					}

					// we'll cap this 'path' with the goals location
					path[ a ].push( aCell.surroundingCells[ i ] );
					i=8;
					
				} else {

					// we'll increment another time since we haven't reached the goal.
					newPath.push( aCell.surroundingCells[ i ] );
					path.push( newPath );

					// cause another cycle of the funciton.
					aa++;
				}
			} 
		}
	}
}

function updateCell( cells , bool , array ){

	// we need to update any cell that has this wall apart of it's surroundings
	// since we allow the AI to travel diagonally we have to check more conditions 
	// than if we only allowed left/right up/down movement.

	switch( cells.info.side ){
		case 0:

			populateValuesForUpdateCell( cells , array , 0 , 4 , 2 , 6 , 1 , 3 , 5 , 7 , bool );
			break;

		case 1:

			populateValuesForUpdateCell( cells , array  , 2 , 6 , 0 , 4 , 1 , 7 , 5 , 3 , bool );
			break;

		case 2:

			populateValuesForUpdateCell( cells , array  , 4 , 0 , 2 , 6 , 3 , 1 , 7 , 5 , bool );
			break;

		case 3:

			populateValuesForUpdateCell( cells , array  , 6 , 2 , 0 , 4 , 7 , 1 , 3 , 5 , bool );
			break;
	}
}

function populateValuesForUpdateCell( cells , array , a , b , c , d , w , x , y , z , bool ){

	var aCell = cells.list[ cells.info.cellID ], bCell, cCell, dCell, eCell, fCell;

	aCell[ array ][ a ] = bool;

	bCell = cells.list[ aCell.surroundingCells[ a ] ];
	bCell[ array ][ b ] = bool;

	cCell = cells.list[ aCell.surroundingCells[ w ] ];
	dCell = cells.list[ aCell.surroundingCells[ z ] ];
	eCell = cells.list[ aCell.surroundingCells[ c ] ];
	fCell = cells.list[ aCell.surroundingCells[ d ] ];

	// check corners and look for a previous wall.  If one is found update all four cells for each
	// corner with false, otherswise, set the travelTo state to true.
	// 
	// 		| b | c |   |
	//		| a | x |   | 
	// 		|   |   |   |
	//
	// 		!( x -> a ) || !( a -> b ) || !( b -> c ){
	//			x->b && b->x && a->c && c->a = false	
	//		} else { 
	//			x->b && b->x && a->c && c->a = true	
	//		} 	

	if( !aCell[ array ][ a ] || !bCell[ array ][ c ] || !cCell[ array ][ b ] ){
		aCell[ array ][ w ] = false;
		bCell[ array ][ x ] = false;
		cCell[ array ][ y ] = false;
		eCell[ array ][ z ] = false;
	} else {
		aCell[ array ][ w ] = true;
		bCell[ array ][ x ] = true;
		cCell[ array ][ y ] = true;
		eCell[ array ][ z ] = true;

	}
	if( !aCell[ array ][ a ] || !bCell[ array ][ d ] || !dCell[ array ][ b ] ){
		aCell[ array ][ z ] = false;
		bCell[ array ][ y ] = false;
		dCell[ array ][ x ] = false;
		fCell[ array ][ w ] = false;
	} else {
		aCell[ array ][ z ] = true;
		bCell[ array ][ y ] = true;
		dCell[ array ][ x ] = true;
		fCell[ array ][ w ] = true;
	}
}